@@ -1,7 +1,7 @@
 //**********************************************************************
 //**********************************************************************
 //
-//  RANDOM SURVIVAL FOREST 3.2.3
+//  RANDOM SURVIVAL FOREST 3.5.0
 //
 //  Copyright 2008, Cleveland Clinic Foundation
 //
@@ -57,31 +57,10 @@
 #ifndef RSFSPLIT_H
 #define RSFSPLIT_H
 #include "extern.h"
-char logRankRandom(Node    *parent,
-                   uint    *splitParameterMax,
-                   double  *splitValueMax);
-char randomSplit(Node    *parent,
-                 uint    *splitParameterMax,
-                 double  *splitValueMax);
-char logRankApprox(Node    *parent,
-                   uint    *splitParameterMax,
-                   double  *splitValueMax);
-char logRankScore(Node    *parent,
-                  uint    *splitParameterMax,
-                  double  *splitValueMax);
-char conserveEvents(Node    *parent,
-                    uint    *splitParameterMax,
-                    double  *splitValueMax);
-char logRank(Node    *parent,
-             uint    *splitParameterMax,
-             double  *splitValueMax);
-uint selectRandomCovariates(Node    *parent,
-                            uint     nodeSize,
-                            uint    *nodeIndex,
-                            uint    *covariateIndex,
-                            double **permissibleSplit,
-                            uint    *permissibleSplitSize);
-uint getSelectableElement(uint    length,
-                          char   *permissible,
-                          double *weight);
+char randomSplit(Node *parent, uint *splitParameterMax);
+char logRankScore(Node *parent,
+                  uint *splitParameterMax);
+char conserveEvents(Node *parent,
+                    uint *splitParameterMax);
+char logRank(Node *parent, uint *splitParameterMax);
 #endif
